
/*! \file dngutils.c
 *  \brief Utilities Driver.
 *         3ds Library.
 *
 *  \version 0.95
 *
 *  3ds image lib using lodepng
 *
 *   Pituka - 3DS Port
 *  (c) Copyright 2016 David Colmenero (aka D_Skywalk)
 */
#include <stdlib.h>
#include <string.h>

#include <3ds.h>

#include "lodepng.h"
#include "../global.h"
#include "imgu.h"

#define simple_RGB565(r,g,b) (unsigned short) ((r << 11)|(g << 5)|b)

void clearTopScreen(void) {
    u8 *frame = gfxGetFramebuffer(GFX_TOP, GFX_LEFT, NULL, NULL);
    memset(frame, 0, 400 * 240 * sizeof(u16));

    // Flush and swap framebuffers
    gfxSwapBuffers();
    gfxFlushBuffers();

    gspWaitForVBlank();
}

void clearBottomScreen(void) {
    u8 *frame = gfxGetFramebuffer(GFX_BOTTOM, GFX_LEFT, NULL, NULL);
    memset(frame, 0, 320 * 240 * sizeof(u16));

    // Flush and swap framebuffers
    gfxSwapBuffers();
    gfxFlushBuffers();

    gspWaitForVBlank();
}

/*
 * Image Wii Functions
 */

/*! \fn void InitImg (t_img * img)
    \brief Inicializa la estructura img.

    \param img imagen a inicializar, DEBE ESTAR LIBERADA.
    \note La funcion no comprueba si el buffer esta liberado o no.
*/
void InitImg(t_img * img) {
    img->Width = 0;
    img->Height = 0;
    img->buffer = NULL;
}

/*! \fn void FreeImg (t_img * img)
    \brief Libera y deja en valores nulos la imagen.

    \param img imagen a liberar.
*/
void FreeImg(t_img * img) {

    if (img->buffer == NULL)
        return;

    free(img->buffer);
    InitImg(img);

}

/*! \fn bool LoadTexture (t_img * img, const unsigned char my_png[])
    \brief Carga un PNG en la estructura img.

    \param img Imagen a usar, DEBE ESTAR LIBERADA.
    \param png_buffer Buffer del png a decodificar.
    \param png_size tamaño del buffer.

    \return true, si todo fue correcto.
    \todo La funcion no comprueba si el buffer esta liberado o no.
*/
bool LoadIMG(t_img * p_oImg, const u8 *png_buffer, const u32 png_size) {
    u8 * encoded = NULL;
    unsigned error = 0xFF;

    error = lodepng_decode24(&encoded, &p_oImg->Width, &p_oImg->Height,
                             png_buffer, png_size);

    if (error)
        return false;

    if (NULL == (p_oImg->buffer = malloc(p_oImg->Width * p_oImg->Height * sizeof(u16)))) {
        return false;
    }

    u16 *pBuffer = (u16*) p_oImg->buffer;
    int i = p_oImg->Width * p_oImg->Height;
    while (i--)
        pBuffer[i] = simple_RGB565((encoded[i * 3] >> 3), (encoded[i * 3 + 1] >> 2), (encoded[i * 3 + 2] >> 3));

    free(encoded);

    return true;
}


void DrawIMG(gfxScreen_t screen, t_img * img) {
    unsigned short *dest = NULL, *src = NULL;
    u8* fb = gfxGetFramebuffer(screen, GFX_LEFT, NULL, NULL);

    dest = (unsigned short *) fb;
    src = (unsigned short *) img->buffer;

    memcpy(dest, src, img->Width * img->Height * sizeof(u16));

    // Flush and swap framebuffers
    gfxSwapBuffers();
    gfxFlushBuffers();

    gspWaitForVBlank();
}

void DrawGFX(gfxScreen_t screen, t_img * img, Goo_Rect *org, Goo_Rect *dest) {
    int size = org->w;
    int loop = org->h;
    int despPerLoop = HRES_3DS; // 240

    unsigned short *ptrDest = NULL, *ptrSrc = NULL;
    u8* fb = gfxGetFramebuffer(screen, GFX_LEFT, NULL, NULL);

    ptrDest = (unsigned short *) fb + (dest->x * HRES_3DS) + (HRES_3DS - dest->y - img->Width);
    ptrSrc = (unsigned short *) img->buffer + (org->x * img->Width);

    while (loop--) {
        memcpy(ptrDest, ptrSrc, size * sizeof(u16));
        ptrSrc += size;
        ptrDest += despPerLoop;
    }
}

// Flush and swap framebuffers
void FlushGFX(void) {
    gfxSwapBuffers();
    gfxFlushBuffers();

    gspWaitForVBlank();
}

void DrawRect(gfxScreen_t screen, t_img * img, Goo_Rect *org, Goo_Rect *dest) {
    DrawGFX(screen, img, org, dest);
    FlushGFX();
}
